/*
  Vibe-O-Matic 3000: Play sounds, likes, and motion like a child car seat in a car.
  Nathan Seidle
  SparkFun Electronics
  August 15th, 2017

  When the rocker is in the 'Car' position and user presses the red button:
  * Play MP3
  * Vibe the motor
  * Control the LEDs

  Based on MP3 Trigger from SparkFun hookup guide: https://learn.sparkfun.com/tutorials/mp3-player-shield-hookup-guide-v15
  LED control is based on Cylon Example from FastLED library: https://github.com/FastLED/FastLED

*/

#include <SPI.h>           // SPI library
#include <SdFat.h>         // SDFat Library
#include <SFEMP3Shield.h>  // MP3 Shield Library

#define terminal Serial

SdFat sd; // Create object to handle SD functions
SFEMP3Shield MP3player; // Create Mp3 library object

#include "FastLED.h" //From https://github.com/FastLED/FastLED

//Define pins
#define BUTTON_PIN A0
#define GND_PIN A1
#define SWITCH_PIN A2
#define DATA_PIN 5
#define CLOCK_PIN 4
#define SHAKER_PIN 10

#define NUM_LEDS 100
CRGB leds[NUM_LEDS];

#define TRACK_TITLE "baby.mp3"

#define MAX_MOTOR_AMOUNT 200 //0 to 255 PWM value. In testing 255 was too noisy and scary.

long randomTimeBetweenPassing = random(1, 10) * 1000; //Pick new random amount between passings
long timeSinceLastPassing = 0;

long randomTimeBetweenMotor = 2000; //Pick new random amount between motor changes
long timeSinceLastMotor = millis();

void setup()
{
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  pinMode(SWITCH_PIN, INPUT_PULLUP);
  pinMode(GND_PIN, OUTPUT);
  digitalWrite(GND_PIN, LOW);

  pinMode(SHAKER_PIN, OUTPUT);
  digitalWrite(SHAKER_PIN, LOW); //Turn off vibe motor

  pinMode(A3, INPUT);
  randomSeed(analogRead(A3));

  terminal.begin(9600);
  terminal.println("Vibe-o-Matic 3000");

  setPwmFrequency(10, 1024); //No noise, full range
  //setPwmFrequency(10, 1); //No noise, 200-255. Seconds work

  initSD();  // Initialize the SD card
  initMP3Player(); // Initialize the MP3 Shield

  //Start talking to the LEDs
  LEDS.addLeds<APA102, DATA_PIN, CLOCK_PIN, RGB>(leds, NUM_LEDS);
  LEDS.setBrightness(84);
}

void loop()
{
  //If user presses the button, check state of switch
  if (isButtonPressed() == true)
  {
    while (isButtonPressed() == true) ; //Wait for you to stop pressing button!

    if (isSwitchOn() == true)
    {
      terminal.println("Start car");
      startCar(); //Car mode
    }
    else
    {
      terminal.println("Start shake");
      startVibe(); //Vibe-only mode
    }
  }
}

//Start playing audio, start playing LEDs, start vibrating
//Run for the length of the audio track or when user presses button
void startCar()
{
  int dice;

  if (MP3player.isPlaying()) MP3player.stopTrack(); //Stop whatever is playing

  //Begin playing
  byte result = MP3player.playMP3(TRACK_TITLE);
  if (result != 0)  // playTrack() returns 0 on success
    terminal.println("Error playing track");
  else
    terminal.println("Track started");

  while (1)
  {
    //If user presses button or audio track is finished, then shut down and return
    if (isButtonPressed() == true || MP3player.isPlaying() == false)
    {
      terminal.println("Stopped car");
      analogWrite(SHAKER_PIN, 0); //Stop Motor
      if (MP3player.isPlaying()) MP3player.stopTrack(); //Stop whatever is playing
      clearLEDs(); //Clear board with a slow fade

      while (isButtonPressed() == true) ; //Wait for you to stop pressing button!
      return;
    }

    //Every 5 seconds check to see if a car randomly passed
    if (millis() - timeSinceLastPassing > randomTimeBetweenPassing)
    {
      //Pick a random light car passing pattern
      dice = random(0, 100);
      if (dice <= 25)
        carPassingLeft();
      else if (dice <= 50)
        carPassingFrontLeftRight();
      else if (dice <= 75)
        carPassingRight();
      else
        carPassingFrontRightLeft();

      randomTimeBetweenPassing = random(1, 10) * 1000; //Pick new random amount between passings
      timeSinceLastPassing = millis();
    }

    //Every 20-40 seconds check to see if we should turn on or off the motor
    if (millis() - timeSinceLastMotor > randomTimeBetweenMotor)
    {
      dice = random(0, 100);
      if (dice <= 50)
      {
        //Pick a random motor speed
        dice = random(100, MAX_MOTOR_AMOUNT);
        analogWrite(SHAKER_PIN, dice);
        terminal.print("motor set to: ");
        terminal.println(dice);
      }
      else
      {
        //Turn off motor
        analogWrite(SHAKER_PIN, 0);
        terminal.println("motor off");
      }

      randomTimeBetweenMotor = random(20, 40) * 1000; //Pick new random amount between passings
      timeSinceLastMotor = millis();

      terminal.print("next motor in ");
      terminal.print(randomTimeBetweenMotor / 1000);
      terminal.println(" seconds");
    }
  }
}

//Vibrate for 20 minutes
void startVibe()
{
  clearLEDs(); //Clear board with a slow fade

  if (MP3player.isPlaying()) MP3player.stopTrack(); //Stop whatever is playing

  analogWrite(SHAKER_PIN, MAX_MOTOR_AMOUNT); //Start motor

  //Go for twenty minutes
  timeSinceLastMotor = millis();
  while (millis() - timeSinceLastMotor < 1000L * 60L * 20L)
  {
    delay(10);
    if (isButtonPressed() == true) break; //Bail if user tells us to
  }

  analogWrite(SHAKER_PIN, 0); //Stop Motor
  terminal.println("Stopped vibe");
}

//Starting at head, run down left side of board
void carPassingLeft()
{
  static uint8_t hue = 0;
  for (int i = NUM_LEDS / 2; i >= 0 ; i--)
  {
    leds[i] = CHSV(hue++, 255, 255);
    FastLED.show();
    fadeAll();

    if (isButtonPressed() == true) return; //Stop if user is pressing button
    delay(10);
  }

  clearLEDs(); //Clear board with a slow fade
}

//Starting at head, run down right side of board
void carPassingRight()
{
  static uint8_t hue = 0;
  for (int i = NUM_LEDS / 2 ; i < NUM_LEDS ; i++)
  {
    leds[i] = CHSV(hue++, 255, 255);
    FastLED.show();
    fadeAll();

    if (isButtonPressed() == true) return; //Stop if user is pressing button
    delay(10);
  }

  clearLEDs(); //Clear board with a slow fade
}

//Starting at mid-left, run down front to mid-right
void carPassingFrontLeftRight()
{
  static uint8_t hue = 0;
  for (int i = NUM_LEDS / 4 ; i < NUM_LEDS / 4 * 3 ; i++)
  {
    leds[i] = CHSV(hue++, 255, 255);
    FastLED.show();
    fadeAll();

    if (isButtonPressed() == true) return; //Stop if user is pressing button
    delay(10);
  }

  clearLEDs(); //Clear board with a slow fade
}

//Starting at mid-right, run down front to mid-left
void carPassingFrontRightLeft()
{
  static uint8_t hue = 0;
  for (int i = NUM_LEDS / 4 * 3 ; i >= NUM_LEDS / 4 ; i--)
  {
    leds[i] = CHSV(hue++, 255, 255);
    FastLED.show();
    fadeAll();

    if (isButtonPressed() == true) return; //Stop if user is pressing button
    delay(10);
  }

  clearLEDs(); //Clear board with a slow fade
}

//Checks to see if user is pressing button
boolean isButtonPressed()
{
  if (digitalRead(BUTTON_PIN) == LOW)
  {
    delay(100); //Debounce
    if (digitalRead(BUTTON_PIN) == LOW)
    {
      return (true);
    }
  }
  return (false);
}

//Checks to see if switch is flipped
boolean isSwitchOn()
{
  if (digitalRead(SWITCH_PIN) == HIGH)
    return (true);
  return (false);
}

//Clears any remaining on LEDs with fade
void clearLEDs()
{
  for (int i = 0 ; i < NUM_LEDS + 15 ; i++)
  {
    FastLED.show();
    fadeAll(); //Finish fade

    if (isButtonPressed() == true) return; //Stop if user is pressing button
    delay(10);
  }
}

//Clears all LEDs with fade
void fadeAll()
{
  for (int i = 0; i < NUM_LEDS; i++)
  {
    leds[i].nscale8(250);
  }
}

// initSD() initializes the SD card and checks for an error.
void initSD()
{
  //Initialize the SdCard.
  if (!sd.begin(SD_SEL, SPI_HALF_SPEED))
    terminal.println("SD failed to init");
  else
    terminal.println("SD card found");

  if (!sd.chdir("/"))
    sd.errorHalt("sd.chdir");
}

// initMP3Player() sets up all of the initialization for the
// MP3 Player Shield. It runs the begin() function, checks
// for errors, applies a patch if found, and sets the volume/
// stero mode.
void initMP3Player()
{
  uint8_t result = MP3player.begin(); // init the mp3 player shield
  MP3player.setVolume(0, 0); //0 is max, 255 is off
  MP3player.setMonoMode(1); // Mono setting 0=off, 3=max
}

//https://playground.arduino.cc/Code/PwmFrequency
void setPwmFrequency(int pin, int divisor) {
  byte mode;
  if (pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch (divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if (pin == 5 || pin == 6) {
      TCCR0B = TCCR0B & 0b11111000 | mode;
    } else {
      TCCR1B = TCCR1B & 0b11111000 | mode;
    }
  } else if (pin == 3 || pin == 11) {
    switch (divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x07; break;
      default: return;
    }
    TCCR2B = TCCR2B & 0b11111000 | mode;
  }
}
